@@ -7,7 +7,7 @@ use strict;
 
 package Log::Report::Dispatcher;
 use vars '$VERSION';
-$VERSION = '0.28';
+$VERSION = '0.90';
 
 
 use Log::Report 'log-report', syntax => 'SHORT';
@@ -186,8 +186,7 @@ sub translate($$$)
     $text .= "\n";
 
     if($show_loc)
-    {   if(my $loc = exists $opts->{location} ? $opts->{location}
-                   : [ $self->collectLocation ])
+    {   if(my $loc = $opts->{location} || $self->collectLocation)
         {   my ($pkg, $fn, $line, $sub) = @$loc;
             # pkg and sub are missing when decoded by ::Die
             $text .= " "
@@ -198,9 +197,7 @@ sub translate($$$)
     }
 
     if($show_stack)
-    {   my $stack
-          = $opts->{stack} ||= $self->collectStack;
-
+    {   my $stack = $opts->{stack} ||= $self->collectStack;
         foreach (@$stack)
         {   $text .= $_->[0] . " "
               . __x( 'at {filename} line {line}'
@@ -217,7 +214,7 @@ sub translate($$$)
 
 
 sub collectStack($)
-{   my ($self, $max) = @_;
+{   my ($thing, $max) = @_;
 
     my ($nest, $sub) = (1, undef);
     do { $sub = (caller $nest++)[3] }
@@ -235,7 +232,7 @@ sub collectStack($)
     {   my ($pkg, $fn, $linenr, $sub) = caller $nest++;
         defined $pkg or last;
 
-        my $line = $self->stackTraceLine(call => $sub, params => \@DB::args);
+        my $line = $thing->stackTraceLine(call => $sub, params => \@DB::args);
         push @stack, [$line, $fn, $linenr];
     }
 
@@ -256,7 +253,7 @@ sub collectLocation()
     @args = caller $nest++
         if +(caller $nest)[3] =~ m/^Log\:\:Report\:\:[^:]*$/;
 
-    @args;
+    @args ? \@args : undef;
 }
 
 
